/**
 * 
 */
package net.bigeon.gclc.swt;

import org.eclipse.swt.widgets.Text;

import net.bigeon.gclc.manager.ConsoleOutput;
import net.bigeon.gclc.utils.AOutputForwardRunnable;
import net.bigeon.gclc.utils.PipedConsoleOutput;

/** @author Emmanuel Bigeon */
public final class ConsoleOutputManager {
    /** The local implementation of the forwarding runnable
     *
     * @author Emmanuel Bigeon */
    private final class ToSWTConsoleForwardRunnable extends AOutputForwardRunnable {
        /** The running status */
        private boolean                  running = true;
        private final PipedConsoleOutput out;

        /** @param manager the manager */
        public ToSWTConsoleForwardRunnable(final PipedConsoleOutput manager) {
            super(manager);
            out = manager;
        }

        @Override
        protected void forwardLine(final String m) {
            appendConsoleOutput(System.lineSeparator() + m);
        }

        @Override
        protected boolean isRunning() {
            return running && !text.isDisposed();
        }

        /** @param running the running to set */
        public void setRunning(final boolean running) {
            this.running = running;
        }

        /** @return the currently forwarded output */
        public PipedConsoleOutput getOuput() {
            return out;
        }
    }

    private final Text                  text;
    private ConsoleOutput               output;
    /** The forwarding runnable */
    private ToSWTConsoleForwardRunnable forward;
    private Thread                      forwardThread;

    /** @param text the text to display the output in. */
    public ConsoleOutputManager(final Text text) {
        super();
        this.text = text;
    }

    /** @param next the next message */
    public void appendConsoleOutput(final String next) {
        text.getDisplay().syncExec(new Runnable() {
            @SuppressWarnings("synthetic-access")
            @Override
            public void run() {
                text.append(next);
            }
        });
    }

    /** Set the output.
     *
     * @param output the output to set */
    public void setManager(final PipedConsoleOutput output) {
        if (forward != null) {
            if (forward.getOuput() == output) {
                return;
            }
            forward.setRunning(false);
        }
        if (output == null) {
            forward = null;
            forwardThread = null;
            return;
        }
        forward = new ToSWTConsoleForwardRunnable(output);
        forwardThread = new Thread(forward, "gclcToSWT"); //$NON-NLS-1$
        forwardThread.start();
    }

    /** @return the forwardThread */
    public Thread getForwardThread() {
        return forwardThread;
    }
}
