/**
 * 
 */
package net.bigeon.gclc.swt;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.io.IOException;

import org.eclipse.swt.widgets.Text;
import org.junit.Test;

import net.bigeon.gclc.utils.PipedConsoleInput;

/** @author Emmanuel Bigeon */
public class ConsoleInputManagerTest {

    private final Text                text = mock(Text.class);
    private final ConsoleInputManager cim  = new ConsoleInputManager(text);

    /** Test method for
     * {@link net.bigeon.gclc.swt.ConsoleInputManager#setInput(java.lang.String)}. */
    @Test
    public void testSetText() {
        cim.setInput("Text");
        verify(text).setText("Text");
    }

    /** Test method for
     * {@link net.bigeon.gclc.swt.ConsoleInputManager#validateInput()}.
     * 
     * @throws IOException if an error occurred */
    @Test
    public void testValidateInput() throws IOException {
        // Expected calls
        when(text.getText()).thenReturn("abc");

        try {
            cim.validateInput();
            fail("No input should cause exception for input validation");
        } catch (final Exception e) {
            // ok
        }
        final PipedConsoleInput input = mock(PipedConsoleInput.class);
        cim.setManager(input);

        cim.validateInput();
        verify(input).type("abc");
    }

    @Test
    public void testValidateInputWithError() throws IOException {
        final PipedConsoleInput input = mock(PipedConsoleInput.class);
        cim.setManager(input);
        // Expected calls
        when(text.getText()).thenReturn("abc");
        doThrow(new IOException()).when(input).type("abc");

        try {
            cim.validateInput();
            fail("IO exception should be forwarded");
        } catch (final IOException e) {
            // ok
        }
    }

    /** Test method for
     * {@link net.bigeon.gclc.swt.ConsoleInputManager#getText()}. */
    @Test
    public void testGetText() {
        assertEquals("Text component should be preserved", text, cim.getText());
    }

    @Test
    public void testGetInput() {
        when(text.getText()).thenReturn("abc");
        assertEquals("Text component should be preserved", "abc", cim.getInput());
    }

}
